/*
    This program demonstrates a couple of ideas about argc and argv.

    First of all, argv is a pointer to a pointer. Since argv is a
    name for an array of strings, so argv is a pointer to an array of
    strings. But a string is a pointer to an array of chars, so argv
    is a pointer to an array of pointers.

    You can also think of argv as a two-dimensional array, since in C
    a two-dimensional array is an array of arrays (but argv is not a
    rectangular two-dimensional array!). So you can access characters
    from the command line using two-dimensional array notation.

    Notice that we know one dimension of argv, that's argc. But what
    about the other dimensions of argv?
*/
#include <stdio.h>   // for printf()
#include <string.h>  // for strlen()

int main(int argc, char* argv[])
{
   int i, j;
   char* p;

   printf("The addresses and contents of argv are:\n");
   for (i = 0; i < argc; i++)
      printf("address %p contains %p\n", argv+i, *(argv+i));

   printf("\n");  // print a blank line

   // print out the command line one character at a time
   for (i = 0; i < argc; i++)  // walk through the array of strings
   {
      j = 0;
      // now walk along one of the parameters (i.e., strings)
      printf("argv[%d]\n", i);
      while( argv[i][j] != 0 )  // strings are terminated by a null
      {
         printf( "   address %p contains %c\n", (*(argv+i))+j,
                                                *((*(argv+i))+j) );
         j++;
      }
   }

   printf("\n"); // print a blank line

/*
    Notice, by looking at the output of this program, that the
    strings are stored first in memory, followed by the array of
    pointers to the strings. There is a small gap between each of
    the strings. What is in these gaps? Let us print out every
    memory location from the beginning of the first string to one
    location past the end of the last string, looking at the contents
    of each location.
*/
   for (p = *argv-1; p < *(argv+argc-1)+strlen(argv[argc-1])+1; p++)
      printf("address %p contains %c or ASCII code %d\n", p, *p, *p);

   return 0;
}//main
